Dbox "Marker Manager"
    // Written by Peter Van Demark - Caliper Corporation - 12/01

    // Manages marker files. You can open an existing marker file, save chosen
    // markers to a new file, delete chosen markers from the opened file, and
    // append chosen markers to an existing file.

    // Using this resource file you can save you favorite markers, send them
    // to others, and they can append the markers to their caliper.mrk file in
    // their Windows folder (e.g., C:|WINNT).

    // You can also use this resource file to learn about reading arrays from
    // and writing arrays to a file, and managing multi-dimensional arrays.

    // To run this dbox using the Maptitude GISDK toolbox:
    // 1. Choose Tools-Add-Ins and open the GIS Developer's Kit toolbox.
    // 2. Compile this file in test mode using the first button (Compile).
    // 3. Run this dbox by clicking the second button (Test), typing the dbox 
    //    name "Marker Manager", clicking the Dialog Box radio button, and
    //    clicking OK.

    // Display the list of markers
    Scroll List "Markers" 1,.6,28,10 List: mrk[1] Multiple Variable: mrk_idx

    // Open a marker file
    Button "Open" 31,1,9 do
	    on error goto quit
	    on escape goto quit
        // Get the name of the marker file
        in_fname = ChooseFile({{"Marker (*.mrk)","*.mrk"}},"Open Marker File", )
	    if in_fname <> null then do
            SetCursor("Hourglass")
             // Read the markers from the file
             mrk = LoadArray(in_fname)
	        end
            // Update the array of indices
	    mrk_idx = null
	    quit: ResetCursor()
    EndItem

    // Save the chosen markers to a new marker file
    Button "Save" same,2.6,9 do
	    if mrk_idx = null then ShowMessage("No markers chosen!") else do
	        on error goto quit
	        on escape goto quit
            // Get the name of the new marker file
            out_fname = ChooseFileName({{"Marker (*.mrk)","*.mrk"}},
                "Save Markers As", )
	        if out_fname <> null then do
                SetCursor("Hourglass")
                dim out_mrk[2]
                n = ArrayLength(mrk_idx)
                // Create an array of the chosen markers
                for i = 1 to n do
                    out_mrk[1] = out_mrk[1] + {mrk[1][mrk_idx[i]]}
                    out_mrk[2] = out_mrk[2] + {mrk[2][mrk_idx[i]]}
                    end
                // Write the chosen markers to the file
                SaveArray(out_mrk,out_fname)
	            end
	        quit: ResetCursor()
            end
    EndItem

    // Delete the chosen markers from the opened file
    Button "Delete" same,4.2,9 do
	    if mrk_idx = null then ShowMessage("No markers chosen!") else do
	        button = MessageBox("Delete chosen markers from " + in_fname + "?",
	        {{"Icon","Warning"}, {"Buttons","YesNo"}})
	        if button = "Yes" then do
                SetCursor("Hourglass")
                // Create array for marker status
                not_mrk = null
                for i = 1 to ArrayLength(mrk[1]) do
                    not_mrk = not_mrk + {True}
                    end
                // Indicate markers to delete
                for i = 1 to ArrayLength(mrk_idx) do
                    not_mrk[mrk_idx[i]] = False
                    end
                // Create array of markers to keep
                dim out_mrk[2]
                for i = 1 to ArrayLength(mrk[1]) do
                    if not_mrk[i] = True then do
                        out_mrk[1] = out_mrk[1] + {mrk[1][i]}
                        out_mrk[2] = out_mrk[2] + {mrk[2][i]}
                        end
                    end
                // Write the markers to the file
                SaveArray(out_mrk,in_fname)
                // Update the arrays of markers and indices
                mrk = out_mrk
	            mrk_idx = null
    	        ResetCursor()
                end
            end
    EndItem

    // Append the chosen markers to the existing file
    Button "Append" same,5.8,9 do
	    if mrk_idx = null then ShowMessage("No markers chosen!") else do
	        on error goto quit
	        on escape goto quit
            // Get the name of the marker file
            out_fname = ChooseFile({{"Marker (*.mrk)","*.mrk"}},"Append Markers To", )
	        if out_fname <> null then do
                SetCursor("Hourglass")
                 // Read the markers from the file
                out_mrk = LoadArray(out_fname)
                n = ArrayLength(mrk_idx)
                // Add the chosen markers to the end of the file
                for i = 1 to n do
                    out_mrk[1] = out_mrk[1] + {mrk[1][mrk_idx[i]]}
                    out_mrk[2] = out_mrk[2] + {mrk[2][mrk_idx[i]]}
                    end
                // Write the markers to the file
                SaveArray(out_mrk,out_fname)
	            end
	        quit: ResetCursor()
	        end
    EndItem

    // Close the Marker Manager
    Button "Close" same,7.4,9 do
        Return()
    EndItem

    Button "Help" same,9,9 do
        MessageBox("Open: opens a marker file.\n\n" +
            "After choosing one or more markers in the scroll list:\n" +
            "  Save: saves the chosen markers to a new file.\n" +
            "  Delete: deletes the chosen markers from the opened file.\n" +
            "  Append: adds the chosen markers to the end of an existing " +
            "file.\n\n" +
            "Close: closes the Marker Manager.",
            {{"Caption","Help for Marker Manager"}})
    EndItem

endDbox
